@@ -115,9 +115,13 @@ hci_dev_up(int sock, gboolean *had_bluetooth) {
 
     if (ioctl(sock, HCIDEVUP, 0) < 0) {
         if (errno == EALREADY) {
+            g_print("[already active] ");
             *had_bluetooth = TRUE;
+            ioctl(sock, HCIDEVDOWN, 0);
+            ioctl(sock, HCIDEVUP, 0);
             return TRUE;
         } else {
+            g_print("[error: %d] ", errno);
             *had_bluetooth = FALSE;
             return FALSE;
         }
@@ -154,12 +158,14 @@ set_i2c_enabled(int sock, gboolean value) {
     opt = 0;
     rv = setsockopt(sock, SOL_HCI, HCI_DATA_DIR, &opt, sizeof(opt));
     if (rv < 0) {
+        g_printf("[socket opt error: %d] ", rv);
         return FALSE;
     }
     addr.hci_family = AF_BLUETOOTH;
     addr.hci_dev = 0;
     rv = bind(sock, (struct sockaddr *) &addr, sizeof(addr));
     if (rv < 0) {
+        g_printf("[socket bind error: %d] ", rv);
         return FALSE;
     }
 
@@ -170,7 +176,7 @@ set_i2c_enabled(int sock, gboolean value) {
 }
 
 
-/* Watching for automatically powering down the radio and I2C when not in use
+/* Watchdog for automatically powering down the radio and I2C when not in use
  * for a while.
  */
 static gboolean
@@ -192,9 +198,14 @@ watchdog_handler(gpointer data) {
         
         g_spawn_command_line_sync("/sbin/rmmod radio-bcm2048",
                                   NULL, NULL, &status, &error);
-        g_print("FM Tuner unloaded\n");
-        return FALSE;
-        
+        if (status == 0) {
+            g_print("FM Tuner unloaded\n");
+            return FALSE;
+        } else {
+            g_printf("FM Tuner in use, cannot unload\n");
+            return TRUE;
+        }
+
     } else {
     
         return TRUE;
@@ -248,7 +259,7 @@ dbus_fmrx_enabler_request(FMRXEnabler *self,
             return TRUE;
         }
         g_print("success\n");
-        
+
         /* power up I2C bus */
         g_print("enabling I2C communication... ");
         if (set_i2c_enabled(self->hci_socket, TRUE) == FALSE) {
@@ -263,7 +274,7 @@ dbus_fmrx_enabler_request(FMRXEnabler *self,
         g_print("success\n");
 
         /* wait for device */
-        usleep(250000);
+        usleep(50000);
 
         /* load driver */
         g_print("loading kernel module... ");
@@ -278,6 +289,8 @@ dbus_fmrx_enabler_request(FMRXEnabler *self,
             if (! self->had_bluetooth) {
                 hci_dev_down(self->hci_socket);
             }
+            g_spawn_command_line_sync("/sbin/rmmod radio-bcm2048",
+                                      NULL, NULL, &status, &error);
             return TRUE;
         }
         g_print("success\n");
@@ -294,6 +307,8 @@ dbus_fmrx_enabler_request(FMRXEnabler *self,
             if (! self->had_bluetooth) {
                 hci_dev_down(self->hci_socket);
             }
+            g_spawn_command_line_sync("/sbin/rmmod radio-bcm2048",
+                                      NULL, NULL, &status, &error);
             return TRUE;
         }
         g_print("success\n");
